/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */

package org.gvsig.mobile.core.auxfunctions;

import java.awt.Container;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * Utility class to read resource files.
 * 
 * @see java.awt.MediaTracker
 * 
 * @author jldominguez
 *
 */
public class ResourceReader {
	
	private static Logger logger = LoggerFactory.getLogger(ResourceReader.class);
	
	/**
	 * number of positions of the GPS cursor
	 */
	public static int GPS_CURSOR_ICON_COUNT = 16;
	public static int IMAGE_LOAD_TIMEOUT = 1200;
	public static String IMG_SKIN_PATH = "";
	/**
	 * Image folder and prefix according to DPI
	 */
	public static String IMG_SKIN_PREFIX = "";
	public static String FILE_SEPARATOR_INSIDE_JAR = "/";

	/**
	 * Used to read images
	 */
	public static MediaTracker mt = new MediaTracker(new Container());
	public static int imageCount = 0;
	
	public static String resourceDir = "";
	
	/**
	 * Gets the GPS icon index  for a given angle
	 * @param angle heading 
	 * @param in_radians whether it is radians or degrees
	 * @return the GPS icon index
	 */
	public static int getGpsIconIndexForAngle(double angle, boolean in_radians) {
		
		double a_in_degrees = (in_radians) ? (angle * 180.0 / Math.PI) : angle;
		
		double step = 360.0 / GPS_CURSOR_ICON_COUNT;
		
		if (Math.abs(a_in_degrees) > 1000) {
			logger.error("Absurd angle: " + angle + "; in_radians = " + in_radians);
			logger.error("Returned North icon (0)");
			return 0;
		}
		
		// normalize
		while (step >= 360) step = step - 360.0;
		while (step < 0) step = step + 360.0;

		int n = Math.round((float) Math.floor(0.5 + a_in_degrees / step));
		
		if ((n < 0) || (n >= GPS_CURSOR_ICON_COUNT)) {
			logger.error("Bad n = " + n + "; GPS_CURSOR_ICON_COUNT = " + GPS_CURSOR_ICON_COUNT);
			logger.error("Forced n = n MOD GPS_CURSOR_ICON_COUNT");
			n = n % GPS_CURSOR_ICON_COUNT;
		}

		logger.debug("-------------- GPS icon index for icon: -----------------");
		logger.debug("angle = " + angle + "; radians = " + in_radians);
		logger.debug("returned: " + n);
		return n;
	}
	

	/**
	 * Gets an image from an array of bytes for a well known format (png, jpg, gif)
	 * @param data the byte array 
	 * @return the image
	 */
	public static Image getImage(byte[] data) {
		Image img = Toolkit.getDefaultToolkit().createImage(data);
		mt.addImage(img,imageCount);
		try {
			mt.waitForID(imageCount, IMAGE_LOAD_TIMEOUT);
			mt.removeImage(img);
			return img;
		} catch (InterruptedException e) {
			logger.error("While downloading: " + e.getMessage());
			return null;
		}
	}

	/**
	 * Gets scaled version of image with Media Tracker.
	 * 
	 * @param img source image
	 * @param newwidth new width
	 * @param newheight new height
	 * @return re sampled image
	 */
	public static Image getScaled(Image img, int newwidth, int newheight) {
		
		Image newimage = img.getScaledInstance(newwidth, newheight, Image.SCALE_FAST);
		mt.addImage(newimage, imageCount);
		try {
			mt.waitForID(imageCount, IMAGE_LOAD_TIMEOUT);
			mt.removeImage(newimage);
			return newimage;
		} catch (InterruptedException e) {
			logger.error("While resampling: " + e.getMessage());
			return null;
		}
	}
	
	/**
	 * Gets an image from a file in a well known format (png, jpg, gif)
	 * @param data the image file 
	 * @return the image object
	 */
	public static Image getImage(File data) {
		try {
			Image img = Toolkit.getDefaultToolkit().createImage(data.getAbsolutePath());
			mt.addImage(img, imageCount);
			mt.waitForID(imageCount, IMAGE_LOAD_TIMEOUT);
			mt.removeImage(img);

			return img;
		} catch (Throwable th) {
			logger.error("While getting downloaded image: " + th.getMessage());
			return null;
		}
	}
	
	/**
	 * Gets an image from the resources folder. It uses the constant IMG_SKIN_PREFIX (prepends it to the
	 * file name)
	 * 
	 * @param dir the resources subfolder
	 * @param incomplete_file_name the file name without the prefix
	 * @return the image from the resources folder
	 */
//	public static Image getResourceImage(String dir, String incomplete_file_name) {
//		String imgPath = resourceDir + File.separator + dir + File.separator + IMG_SKIN_PREFIX + incomplete_file_name;
//		Image img = Toolkit.getDefaultToolkit().createImage(imgPath);
//		mt.addImage(img,imageCount);
//		try{
//			mt.waitForID(imageCount, IMAGE_LOAD_TIMEOUT);
//			mt.removeImage(img);
//			return img;
//		} catch (InterruptedException e) {
//			logger.error("While while getting img with media tracker: " + e.getMessage());
//			return null;
//		}	
//	}
	public static Image getResourceImage(String dir, String incomplete_file_name){
		
		
		String path = dir + FILE_SEPARATOR_INSIDE_JAR + IMG_SKIN_PREFIX + incomplete_file_name;
		URL imgf = null;
		try {
			imgf = new URL(path);
		} catch (MalformedURLException e) {
			logger.error("Bad URL: " + path);
		}
		
		try {
			Image img = Toolkit.getDefaultToolkit().createImage(imgf);
			mt.addImage(img, imageCount);
			mt.waitForID(imageCount, IMAGE_LOAD_TIMEOUT);
			mt.removeImage(img);

			return img;
		} catch (Throwable th) {
			logger.error("While getting downloaded image: " + th.getMessage());
			return null;
		}

		
//		logger.debug("Cargando Imagen: "+ dir + FILE_SEPARATOR_INSIDE_JAR + IMG_SKIN_PREFIX + incomplete_file_name);
//		Image img = new ImageLoader().getImage(dir + FILE_SEPARATOR_INSIDE_JAR + IMG_SKIN_PREFIX + incomplete_file_name);
//		ResourceReader.mt.addImage(img,ResourceReader.imageCount);
//		try{
//			ResourceReader.mt.waitForID(ResourceReader.imageCount, ResourceReader.IMAGE_LOAD_TIMEOUT);
//			ResourceReader.mt.removeImage(img);
//			return img;
//		} catch (InterruptedException e) {
//			logger.error("While while getting img with media tracker: " + e.getMessage());
//			return null;
//		}	
	}
	
	/**
	 * Gets a file from the resource folder 
	 * @param dir
	 * @param file_name
	 * @return the file object
	 */
	public static File getResourceFile(String dir, String file_name) {
		String filePath = resourceDir + File.separator + dir + File.separator + file_name;
		File resp = new File(filePath);
		return resp;
	}
	
	
	public static boolean isDirectory(File f) {
		
		String str = f.getAbsolutePath();
		int lastbar = str.lastIndexOf(File.separator);
		str = str.substring(lastbar+1); 
		return (str.indexOf(".") == -1);
	}
}
