/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Prodevelop and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *   +34 963862235
 *   gvsig@gva.es
 *   http://www.gvsig.gva.es
 *
 *    or
 *
 *   Prodevelop Integracin de Tecnologas SL
 *   Conde Salvatierra de lava , 34-10
 *   46004 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   gis@prodevelop.es
 *   http://www.prodevelop.es
 *
 *    or
 *
 *   Instituto de Robtica
 *   Apartado de correos 2085
 *   46071 Valencia
 *   (Spain)
 *   
 *   +34 963 543 577
 *   jjordan@robotica.uv.es
 *   http://robotica.uv.es
 *   
 */
package org.gvsig.mobile.ui.awt.tools;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import java.util.Hashtable;

import org.zaval.lw.LwComponent;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwLayer;
import org.zaval.lw.LwManager;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.LwWinLayer;
import org.zaval.lw.TooltipInfo;
import org.zaval.lw.event.LwMouseEvent;
import org.zaval.lw.event.LwMouseListener;
import org.zaval.lw.event.LwMouseMotionListener;
import org.zaval.util.Timer;

/**
 * This manager is used to support tool tip for a light weight components. You can use
 * any light weight component as the tool tip. The tool tip component is shown whenever the mouse cursor
 * stays immobile inside the specified owner component area (and none of the mouse buttons have been pressed).
 * There are several ways to support the tool tips for the given light weight component:
 * <ul>
 *   <li>
 *     The light weight component has to implement the TooltipInfo interface.
 *     From this moment the manager will use the implementation to get a tool tip for the component
 *     at the specified location of the target component.
 *   </li>
 *   <li>
 *     The manager provides the <code>setTooltip</code> method that allows you to define the
 *     TooltipInfo interface or a string or a light wight component for the specified component.
 *   </li>
 * </ul>
 */
public class TBMenuManager
implements LwManager, LwMouseListener, LwMouseMotionListener, Runnable
{
  private int         x, y, tick = 400;
  private LwLayer     targetLayer;
  private LwComponent tooltip, target;
  private Hashtable   tooltips;
  
  private static TBMenuManager man;
  
  public static TBMenuManager getInstance() {
	  if(man == null) {
		  man = new TBMenuManager();
	  } 
	  return man;	  
  } 

/**
  * Sets the specified tick (in milliseconds) that defines "frozen" mouse interval
  * after the tool tip will be shown.
  * @param t the specified tick.
  */
  public void setTick (int t) {
    tick = t;
  }

 /**
  * Sets the specified tool tip data for the given component. You can use
  * the following object as the tool tip data:
  * <ul>
  *   <li><b>String instance</b>. In this case the string is used as the content
  *       of a tooltip component. The tooltip component is created automatically.
  *   <li><b>LwComponent instance</b>. In this case the component will be shown as
  *       the tool tip.
  *   <li><b>TooltipInfo instance</b>. In this case the interface will be used to
  *       get a component that should be shown as a tool tip at the specified
  *       location.
  * </ul>
  * Use <code>null</code> as the tool tip data to remove the tool tip for the given
  * component.
  * @param c the component.
  * @param data the specified tool tip data.
  */
  public void setTooltip (LwComponent c, Object data)
  {
    if (data != null)
    {
      if (tooltips == null) tooltips = new Hashtable();
      tooltips.put(c, (data instanceof String)?createTooltip((String)data):data);
    }
    else
    {
      if (target == c)
      {
        Timer.getTimer(false).remove(this);
        target = null;
        hideTooltipInfo();
      }
      tooltips.remove(c);
    }
  }

  public void mouseEntered (LwMouseEvent e)
  {
    LwComponent c = e.getLwComponent ();
    if (c instanceof TooltipInfo || (tooltips != null && tooltips.get(c) != null))
    {
      target      = c;
      targetLayer = LwToolkit.getDesktop(c).getLayer(LwWinLayer.ID);
      this.x = e.getX();
      this.y = e.getY();
      Timer.getTimer(false).add(this, tick, tick);
    }
  }

  public void mouseExited  (LwMouseEvent e)
  {
    if (target != null)
    {
      Timer.getTimer(false).remove(this);
      target = null;
      hideTooltipInfo();
    }
  }

  public void mouseClicked (LwMouseEvent e) {}

  public void mousePressed (LwMouseEvent e)
  {
    if (target != null)
    {
      Timer.getTimer(false).remove(this);
      hideTooltipInfo();
    }
  }

  public void mouseReleased(LwMouseEvent e)
  {
    if (target != null)
    {
      this.x = e.getX();
      this.y = e.getY();
      Timer.getTimer(false).add(this, tick, tick);
    }
  }

  public void startDragged(LwMouseEvent e) {}
  public void endDragged  (LwMouseEvent e) {}
  public void mouseDragged(LwMouseEvent e) {}

  public void mouseMoved  (LwMouseEvent e)
  {
    if (target != null)
    {
      Timer.getTimer(false).clear(this);
      x = e.getX();
      y = e.getY();
      hideTooltipInfo();
    }
  }

  public void run ()
  {
    if (tooltip == null)
    {
      Object tp = tooltips != null?tooltips.get(target):null;
      if (tp == null && target instanceof TooltipInfo) tp = target;
      tooltip = (tp instanceof TooltipInfo)?((TooltipInfo)tp).getTooltip(target, x, y)
                                           :(LwComponent)tp;
      if (tooltip != null)
      {
        Dimension ps = tooltip.getPreferredSize();
        tooltip.setSize(ps.width, ps.height);
        Point p = LwToolkit.getAbsLocation(x, y, target);
        int tx = p.x, ty = p.y - tooltip.getHeight();

        int dw = targetLayer.getWidth();
        if (tx + ps.width  > dw) tx = dw - ps.width - 1;
        tooltip.setLocation (tx < 0?0:tx, target.getY() + target.getHeight());
        
        targetLayer.add(LwWinLayer.INFO_WIN, tooltip);
      }
    }
  }

  public void dispose() {
    if (tooltips != null) tooltips.clear();
  }

 /**
  * Creates and returns a tool tip component by the given label. The method can be
  * used to create standard tool tip component for light weight applications.
  * @param text the specified text.
  * @return the tool tip component.
  */
  public static LwComponent createTooltip(String text)
  {
    LwLabel lab = new LwLabel(new org.zaval.data.Text(text));
    lab.setBackground((Color)LwToolkit.getStaticObj("tt.bgcol"));
    lab.getTextRender().setFont ((Font)LwToolkit.getStaticObj("def.sfn"));
    lab.getViewMan(true).setBorder("br.plain");
    lab.setInsets(1,4,1,4);
    Dimension d = lab.getPreferredSize();
    lab.setSize(d.width, d.height);
    return lab;
  }

  private void hideTooltipInfo()
  {
    if (tooltip != null)
    {
    	try {
    		targetLayer.remove(targetLayer.indexOf(tooltip));
    	} catch (Exception e) {
    		
    	}
      
      tooltip = null;
    }
  }
}


