/**
 *     Caption: J2SE/J2ME(Personal Profile) Light-Weight Visual Components Library
 *
 *     @author:     Andrei Vishnevsky
 *     @version:    4.50
 *
 * The latest product version is always available from the product's homepage:
 * http://www.lwvcl.com/
 *
 * Copyright (C) 2001-2005  Zaval Creative Engineering Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package org.gvsig.mobile.ui.lw;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;
import java.io.File;
import java.io.FilenameFilter;

import org.gvsig.i18n.Messages;
import org.gvsig.mobile.core.auxfunctions.ResourceReader;
import org.gvsig.mobile.ui.awt.WindowManager;
import org.zaval.data.Item;
import org.zaval.data.Tree;
import org.zaval.data.TreeModel;
import org.zaval.lw.LwBorderLayout;
import org.zaval.lw.LwCombo;
import org.zaval.lw.LwCompList;
import org.zaval.lw.LwConstraints;
import org.zaval.lw.LwGridLayout;
import org.zaval.lw.LwLabel;
import org.zaval.lw.LwLayout;
import org.zaval.lw.LwPanel;
import org.zaval.lw.LwScrollPan;
import org.zaval.lw.LwTextField;
import org.zaval.lw.LwToolkit;
import org.zaval.lw.event.LwActionListener;
import org.zaval.lw.event.LwActionSupport;
import org.zaval.lw.event.LwChildrenListener;
import org.zaval.lw.event.LwKeyEvent;
import org.zaval.lw.event.LwKeyListener;
import org.zaval.lw.event.LwVCLEvent;
import org.zaval.lw.misc.LwCustomTreeEditor;
import org.zaval.lw.misc.LwCustomTreeView;
import org.zaval.lw.tree.LwTree;

/**
 * This panel provides ability to select a file in the files tree hierarchy.
 * Register an action listener to be notified whenever a selected file has been
 * changed.
 * <p>
 * The panel reads fields titles as the following LwVCL resources:
 * <Table border="0">
 * <tr>
 * <td><b>fld.path</b></td>
 * <td>path field title</td>
 * </tr>
 * <tr>
 * <td><b>fld.mask</b></td>
 * <td>mask combo box title</td>
 * </tr>
 * <tr>
 * <td><b>fld.file</b></td>
 * <td>file field title</td>
 * </tr>
 * </table>
 * </p>
 */
public class FilePanel extends LwPanel implements LwActionListener,
		LwChildrenListener, LwKeyListener {
	private String root;
	private String[][] ext;
	private LwTextField fileField;
	private LwTextField pathField;
	private LwLabel pathFieldLabel;
	private boolean fileFieldEditable = false;
	private LwTree tree;
	private LwCombo filterCombo;
	private LwCombo rootsCombo;
	private LwActionSupport support;
	
	private LwPanel bottomPanel;
	private int wtwothirds = 160;
	
	private IValidStateListener validListener = null;
	private boolean onlyDirs = false;

	/**
	 * Constructs the file panel with the specified root directory. The
	 * constructor setups a filter to list all available files and directories.
	 * 
	 * @param root
	 *            the specified root directory. The directory tree will be
	 *            created relatively the root directory.
	 */
	public FilePanel(String root, boolean onlyDirectories, IValidStateListener vallis) {		
		this(root,
				!onlyDirectories ? new String[][] { { "Show All Files", "*" } }
						: new String[][] { { "", "" } }, vallis);
	}

	/**
	 * Constructs the file panel with the specified root directory and the given
	 * file mask.
	 * 
	 * @param root
	 *            the specified root directory. The directory tree will be
	 *            created relatively the root directory.
	 * @param ext
	 *            the file filter. The filter is a two-dimensional array where
	 *            the first element corresponds to the extension filter title
	 *            and the second element specifies a file extension filter.
	 */
	public FilePanel(String root, String[][] ext, IValidStateListener vallis) {
		
		onlyDirs = (ext == null || (ext[0][0].length() == 0 && ext[0][1].length() == 0));
		
		Dimension mainwi = WindowManager.getInstance().getMainSize();
		if (mainwi != null) {
			wtwothirds = (mainwi.width * 2) / 3;
		}
		
		validListener = vallis;

		if (root == null) root = System.getProperty("user.home");
		this.root = root;
		this.setOpaque(false);
		this.setBackground(Color.white);

		/* C#this.ext = new string[ext.Length][]; */
		/* C#for (int i=0; i<ext.Length; i++) this.ext[i] = new string[2]; */

		this.ext = new String[ext.length][2]; /* java */
		for (int i = 0; i < ext.length; i++)
			for (int j = 0; j < 2; j++)
				this.ext[i][j] = ext[i][j];

		setInsets(2, 2, 2, 2);
		tree = new LwTree(null);
		LwCustomTreeView _c = new LwCustomTreeView();
		_c.setDefFont(DialogConstants.getDefaultFont());
		tree.setViewProvider(new LwCustomTreeView());
		tree.setEditorProvider(new LwCustomTreeEditor());
		tree.addSelectionListener(this);
		LwScrollPan sp = new LwScrollPan(tree);
		sp.setOpaque(false);
		sp.setBackground(Color.white);
		sp.getViewMan(true).setBorder("br.sunken");
		add(LwBorderLayout.CENTER, sp);

		// ===========================================
		LwConstraints c1 = new LwConstraints();
		c1.ax = LwToolkit.LEFT;
		c1.fill = 0;
		c1.insets = new Insets(3,3,3,3);
		LwConstraints c2 = new LwConstraints();
		c2.ax = LwToolkit.RIGHT;
		c2.fill = LwToolkit.HORIZONTAL;
		c2.insets = new Insets(3,3,3,3);
		LwConstraints c3 = new LwConstraints();
		c3.ax = LwToolkit.LEFT;
		c3.fill = 0;
		c3.insets = new Insets(3,3,3,3);
		LwConstraints c4 = new LwConstraints();
		c4.ax = LwToolkit.RIGHT;
		c4.fill = LwToolkit.HORIZONTAL;
		c4.insets = new Insets(3,3,3,3);
		LwConstraints c5 = new LwConstraints();
		c5.ax = LwToolkit.LEFT;
		c5.fill = 0;
		c5.insets = new Insets(3,3,3,3);
		LwConstraints c6 = new LwConstraints();
		c6.ax = LwToolkit.RIGHT;
		c6.fill = LwToolkit.HORIZONTAL;
		c6.insets = new Insets(3,3,3,3);
		// ===========================================
		
		LwPanel topPanel = new LwPanel();
		topPanel.setOpaque(false);
		topPanel.setBackground(Color.white);
		topPanel.setLwLayout(new LwGridLayout(2, 2, LwToolkit.HORIZONTAL));
		// ==== 
		pathFieldLabel = DialogConstants.getDefaultLabel(Messages
				.getText("AppSettingsDialog.6"));
		pathFieldLabel.setBackground(Color.white);
		// ==== 
		 pathField = DialogConstants.getDefaultTextField();
		 // pathField.setPSSize(80, -1);
		 pathField.setEditable(false);
		// ========
		LwLabel l3 = DialogConstants.getDefaultLabel(Messages
				.getText("AppSettingsDialog.11"));
		l3.setBackground(Color.white);
		// ==== 
		rootsCombo = new LwCombo();		
		LwCompList lC = new LwCompList();
		lC.addItem(DialogConstants.getDefaultLabel(""));
		File[] roots = File.listRoots();
		
		if (roots.length <= 0)		
			roots = new File(System.getProperty("user.dir")).listFiles();		
		
		if (roots.length > 0) lC.remove(0);
		for (int i = 0; i<roots.length; i++) {
			if (ResourceReader.isDirectory(roots[i])) {
				lC.addItem(DialogConstants.getDefaultLabel(roots[i].getAbsolutePath()));
			}
		}
		lC.addSelectionListener(this);		
		rootsCombo.setList(lC);		
		// rootsCombo.setPSSize(80, -1);	
		// =====
		

		// ===========================================
		
		bottomPanel = new LwPanel();
		bottomPanel.setOpaque(false);
		bottomPanel.setBackground(Color.white);
		bottomPanel.setLwLayout(new LwGridLayout(2, 2, LwToolkit.HORIZONTAL));
		
		// ==== 

		LwLabel l1 = DialogConstants.getDefaultLabel((String) LwToolkit
				.getStaticObj("fld.name"));
		l1.setBackground(Color.white);
		
		// ====
		
		fileField = new ListenableTextField("", this);
		// DialogConstants.getDefaultTextField();
		// fileField.setPSSize(70, -1);
		fileField.setEditable(fileFieldEditable);
		
		
		
		// ==== 
			
		LwLabel l = DialogConstants.getDefaultLabel((String) LwToolkit
				.getStaticObj("fld.mask"));
		l.setBackground(Color.white);
		
		// ====
		
		filterCombo = new LwCombo();
		LwCompList list = new LwCompList();
		list.addSelectionListener(this);
		filterCombo.setList(list);
		
		for (int i = 0; i < ext.length; i++)
			list.addItem(DialogConstants.getDefaultLabel(ext[i][0]));
		
		if (list.itemCount() > 0) list.select(0);
		if (lC.itemCount() > 0) lC.select(0);
		
		// filterCombo.setPSSize(70, -1);
		// ==== 

		// =================
		
		if (!onlyDirs) {
			topPanel.add(c5, l);
			filterCombo.setPSSize(wtwothirds, -1);
			topPanel.add(c6, filterCombo);
		}
		
		topPanel.add(c1, l3);
		rootsCombo.setPSSize(wtwothirds, -1);
		topPanel.add(c2, rootsCombo);
		
		add(LwBorderLayout.NORTH, topPanel);
		
		bottomPanel.add(c1, pathFieldLabel);
		pathField.setPSSize(wtwothirds, -1);
		bottomPanel.add(c2, pathField);
		
		bottomPanel.add(c3, l1);
		fileField.setPSSize(wtwothirds, -1);
		bottomPanel.add(c4, fileField);
		
		add(LwBorderLayout.SOUTH, bottomPanel);	
		
//		topPanel.add(c1, l2);
//		topPanel.add(c2, pathField);
//		topPanel.add(c1, l3);
//		topPanel.add(c2, rootsCombo);
//		add(LwBorderLayout.NORTH, topPanel);
//		
//		bottomPanel.add(c3, l1);
//		bottomPanel.add(c4, fileField);
//		bottomPanel.add(c5, l);
//		bottomPanel.add(c6, filterCombo);
//		add(LwBorderLayout.SOUTH, bottomPanel);	
	}

	/**
	 * Gets the tree component that represents the file tree.
	 * 
	 * @return a tree component.
	 */
	public LwTree getFileTree() {
		return tree;
	}

	/**
	 * Gets the selected file.
	 * 
	 * @return a selected file.
	 */
	public File getSelectedFile() {
		FileItem selected = (FileItem) tree.getSelectedItem();
		return selected == null ? null : selected.getFile();
	}

	/**
	 * Search a file by the specified path.
	 * 
	 * @param path
	 *            the specified file path.
	 * @return a file if it has been found, otherwise the method returns
	 *         <code>null</code>.
	 */
	public File lookup(String path) {
		FileItem fi = lookup(tree.getModel(), (FileItem) tree.getModel()
				.getRoot(), path);
		return fi == null ? null : fi.getFile();
	}

	/**
	 * Adds the action listener to get events whenever a new file has been
	 * selected.
	 * 
	 * @param l
	 *            the specified action listener.
	 */
	public void addActionListener(LwActionListener l) {
		if (support == null)
			support = new LwActionSupport();
		support.addListener(l);
	}

	/**
	 * Removes the action listener.
	 * 
	 * @param l
	 *            the specified action listener.
	 */
	public void removeActionListener(LwActionListener l) {
		if (support != null)
			support.removeListener(l);
	}

	public void childPerformed(LwVCLEvent e) {
		if (e.getID() == LwKeyEvent.KEY_PRESSED
				&& e.getSource() == fileField
				&& ((LwKeyEvent) e).getKeyCode() == java.awt.event.KeyEvent.VK_ENTER) {
			FileItem selected = (FileItem) tree.getSelectedItem();
			if (selected != null
					&& !selected.getFileName().equals(fileField.getText())) {
				FileItem item = lookup(tree.getModel(), selected, fileField
						.getText());
				if (item != null)
					tree.select(item);
			}
		}
	}

	public void actionPerformed(Object target, Object data) {
		if (target == tree) {
			FileItem item = (FileItem) data;
			pathField.setText((item == null) ? "" : item.getDir());
			
			if (item != null && !item.isDirectory()) {
				fileField.setText(item.getFileName());
				notifyValid(true);
			} else {
				fileField.setText("");
				notifyValid(false);
			}
			
			if (item != null) {
				//si no tiene hijos los cargamos
				try {
					tree.getModel().getChildAt(item, 0);
				} catch (Exception e) {
					createTreeModel(tree.getModel(), item, item.getFile(), new ExtFileFilter(
							ext[filterCombo.getList().getSelectedIndex()][1]));
				} 
			}
				
			repaint();
			if (support != null) {
				AppSettings.lastPath = pathField.getText();
				support.perform(this, pathField.getText() + File.separator
						+ fileField.getText());
			}
				
		} else if (target == filterCombo.getList()) {
			TreeModel model = createTreeModel(root, new ExtFileFilter(
					ext[filterCombo.getList().getSelectedIndex()][1]));
			tree.setModel(model);
			notifyValid(false);
			repaint();
		} else if (target == rootsCombo.getList()) {
			LwLabel l = (LwLabel)rootsCombo.getList().getSelected();
			String root = l.getText();
			TreeModel model = createTreeModel(root, new ExtFileFilter(
					ext[filterCombo.getList().getSelectedIndex()][1]));
			tree.setModel(model);
			notifyValid(false);
			repaint();
		}
	}

	private void notifyValid(boolean b) {
		
		if (validListener != null) {
			validListener.newState(b);
		}
		
	}

	protected/* C#override */LwLayout getDefaultLayout() {
		return new LwBorderLayout(5, 5);
	}

	private FileItem lookup(TreeModel model, FileItem root, String path) {
		if (root.isDirectory()) {
			int index = path.indexOf(File.separator);
			String s = (index >= 0) ? path.substring(0, index) : path;
			for (int i = 0; i < model.getChildrenCount(root); i++) {
				FileItem fi = (FileItem) model.getChildAt(root, i);
				if (fi.getFileName().equals(s))
					return (index >= 0) ? lookup(model, fi, path
							.substring(index + 1)) : fi;
			}
		}
		return null;
	}

	/**
	 * Creates and returns the tree model basing on the specified root directory
	 * and the given file filter.
	 * 
	 * @param root
	 *            the specified root directory.
	 * @param f
	 *            the specified file filter.
	 * @return a tree model that represents the a hierarchy of the files.
	 */
	public static TreeModel createTreeModel(String root, FilenameFilter f) {

		File rootFile = new File(root);

		if (!rootFile.exists())
			throw new IllegalArgumentException();
		
		FileItem rootItem = new FileItem(rootFile);
		
		TreeModel model = new Tree(rootItem);
		createTreeModelDirectory(model, rootItem, rootFile, f);
		return model;
	}

	private static void createTreeModelDirectory(TreeModel t, FileItem r, File f,
			FilenameFilter filter) {
		if (ResourceReader.isDirectory(f)) {
			String[] dirs = f.list(filter);
			int len = dirs.length;
			for (int i = 0; i < len; i++) {
				File kidFile = new File(r.getDir() + File.separator + dirs[i]);
				FileItem kidItem = new FileItem(kidFile);
				t.add(r, kidItem);								
//				createTreeModel(t, kidItem, kidFile, filter);
			}			
		}
	}
	
	private static void createTreeModel(TreeModel t, FileItem r, File f,
			FilenameFilter filter) {
		if (ResourceReader.isDirectory(f)) {
			String[] dirs = f.list(filter);
			
			if (dirs != null) {
				for (int i = 0; i < dirs.length; i++) {
					File kidFile = new File(r.getDir() + File.separator + dirs[i]);
					FileItem kidItem = new FileItem(kidFile);
					t.add(r, kidItem);
//					createTreeModel(t, kidItem, kidFile, filter);
				}
			}
		} 
	}
	
	public String getFileFieldText() {
		if (fileField == null) {
			return "";
		} else {
			return fileField.getText();
		}
	}
	
	public void setFileFiledEditable(boolean v) {
		fileFieldEditable = v;
		if (fileField != null) {
			fileField.setEditable(fileFieldEditable);
		}
		
	}
	
	private class ListenableTextField extends LwTextField {
		
		private LwKeyListener klist;
		public ListenableTextField(String text, LwKeyListener klis) {
			super(text);
			getTextRender().setFont(DialogConstants.getDefaultFont());
			klist = klis;
		}
		
		public void keyReleased(LwKeyEvent kev) {
			super.keyReleased(kev);
			if (klist != null) {
				klist.keyReleased(kev);
			}
		}
	}

	public void keyPressed(LwKeyEvent arg0) { }
	public void keyReleased(LwKeyEvent kev) {
		if (kev.getSource() == this.fileField) {
			this.notifyValid(validFileName(fileField.getText()));
		}
	}

	public void keyTyped(LwKeyEvent kev) { }

	public static boolean validFileName(String text) {
		
		String lowerc = text.toLowerCase();
		// a-z 0-9 _-
		int sz = lowerc.length();
		for (int i=0; i<sz; i++) {
			if (!validFileNameLowCaseChar(lowerc.charAt(i))) {
				return false;
			}
		}
		return true;
	}

	public static boolean validFileNameLowCaseChar(char ch) {
		
		if (ch >= 'a' && ch <= 'z') return true; 
		if (ch >= '1' && ch <= '9') return true; 
		if (ch == '0') return true; 
		if (ch == '_') return true; 
		if (ch == '-') return true; 
		return false;
	}
}

class FileItem extends Item {
	private String path;
	private boolean isDirectoryValue;

	FileItem(File f) {
		super(f.getName());
		path = f.getPath();
		isDirectoryValue = ResourceReader.isDirectory(f);
		if (!isDirectoryValue) {
			int index = path.lastIndexOf(File.separator);
			path = index < 0 ? "" : path.substring(0, index);
		}
	}

	String getDir() {
		return path;
	}

	String getFileName() {
		return getValue().toString();
	}

	File getFile() {
		return (isDirectory()) ? new File(path) : new File(
				path.length() > 0 ? path + File.separator + getFileName()
						: getFileName());
	}

	boolean isDirectory() {
		return isDirectoryValue;
	}
}

class ExtFileFilter implements FilenameFilter {
	
	private static int count = 0;
	private String ext;

	ExtFileFilter(String ext) {
		this.ext = ext;
	}

	public boolean accept(File dir, String name) {
		
		count++;
		File aux = new File(dir.getPath() + File.separator + name);
		// aux.list()c
		boolean auxb = ResourceReader.isDirectory(aux);
		
		if (ext.equals("*") || auxb)
			return true;

		int index = name.lastIndexOf('.');
		boolean resp = index > 0 && ext.equalsIgnoreCase(name.substring(index + 1)); 
		return resp;
	}
}
